@@ -4,6 +4,7 @@ from twisted.trial import unittest
 from twisted.internet import defer, task
 from twisted.python.failure import Failure
 from zope.interface import implements
+from mock import patch
 
 from txtorcon import Circuit
 from txtorcon import build_timeout_circuit
@@ -99,12 +100,28 @@ class CircuitTests(unittest.TestCase):
 
         circuit = Circuit(tor)
         now = datetime.datetime.now()
-        update = '1 LAUNCHED PURPOSE=GENERAL TIME_CREATED=%s' % time.strftime('%Y-%m-%dT%H:%M:%S')
+        update = '1 LAUNCHED PURPOSE=GENERAL TIME_CREATED=%s' % now.strftime('%Y-%m-%dT%H:%M:%S')
         circuit.update(update.split())
         diff = circuit.age(now=now)
         self.assertEquals(diff, 0)
         self.assertTrue(circuit.time_created is not None)
 
+    @patch('txtorcon.circuit.datetime')
+    def test_age_default(self, fake_datetime):
+        """
+        age() w/ defaults works properly
+        """
+        from datetime import datetime
+        now = datetime.fromtimestamp(60.0)
+        fake_datetime.return_value = now
+        fake_datetime.utcnow = Mock(return_value=now)
+        tor = FakeTorController()
+
+        circuit = Circuit(tor)
+        circuit._time_created = datetime.fromtimestamp(0.0)
+        self.assertEquals(circuit.age(), 60)
+        self.assertTrue(circuit.time_created is not None)
+
     def test_no_age_yet(self):
         """
         make sure age doesn't explode if there's no TIME_CREATED flag.
