@@ -1203,6 +1203,42 @@ class LaunchTorTests(unittest.TestCase):
         # cancel the errback chain, we wanted this
         return None
 
+    @defer.inlineCallbacks
+    def test_launch_tor_unix_controlport(self):
+        config = TorConfig()
+        config.ControlPort = "unix:/dev/null"
+        trans = FakeProcessTransport()
+        trans.protocol = self.protocol
+        fakeout = StringIO()
+        fakeerr = StringIO()
+
+        def connector(proto, trans):
+            proto._set_valid_events('STATUS_CLIENT')
+            proto.makeConnection(trans)
+            proto.post_bootstrap.callback(proto)
+            return proto.post_bootstrap
+
+        def on_protocol(proto):
+            proto.outReceived('Bootstrapped 90%\n')
+
+        reactor = FakeReactor(self, trans, on_protocol)
+        reactor.connectUNIX = Mock()
+        try:
+            yield launch_tor(
+                config,
+                reactor,
+                tor_binary='/bin/echo',
+                stdout=fakeout,
+                stderr=fakeerr
+            )
+        except Exception:
+            pass
+        self.assertTrue(reactor.connectUNIX.called)
+        self.assertEqual(
+            '/dev/null',
+            reactor.connectUNIX.mock_calls[0][1][0],
+        )
+
     def test_launch_tor_fails(self):
         config = TorConfig()
         config.OrPort = 1234
@@ -1611,6 +1647,37 @@ ControlPort Port''')
         process.progress(10, 'tag', 'summary')
         self.assertTrue(self.got_progress)
 
+    def test_quit_process(self):
+        process = TorProcessProtocol(None)
+        process.transport = Mock()
+
+        d = process.quit()
+        self.assertFalse(d.called)
+
+        process.processExited(Failure(error.ProcessTerminated(exitCode=15)))
+        self.assertTrue(d.called)
+        process.processEnded(Failure(error.ProcessDone(None)))
+        self.assertTrue(d.called)
+        errs = self.flushLoggedErrors()
+        self.assertEqual(1, len(errs))
+        self.assertTrue("Tor exited with error-code" in str(errs[0]))
+
+    def test_quit_process_already(self):
+        process = TorProcessProtocol(None)
+        process.transport = Mock()
+
+        def boom(sig):
+            self.assertEqual(sig, 'TERM')
+            raise error.ProcessExitedAlready()
+        process.transport.signalProcess = Mock(side_effect=boom)
+
+        d = process.quit()
+        process.processEnded(Failure(error.ProcessDone(None)))
+        self.assertTrue(d.called)
+        errs = self.flushLoggedErrors()
+        self.assertEqual(1, len(errs))
+        self.assertTrue("Tor exited with error-code" in str(errs[0]))
+
     def test_status_updates(self):
         process = TorProcessProtocol(None)
         process.status_client("NOTICE CONSENSUS_ARRIVED")
