@@ -104,6 +104,17 @@ class AuthenticationTests(unittest.TestCase):
 
         self.assertEqual(self.transport.value(), 'AUTHENTICATE %s\r\n' % "foo".encode("hex"))
 
+    def test_authenticate_null(self):
+        self.protocol.makeConnection(self.transport)
+        self.assertEqual(self.transport.value(), 'PROTOCOLINFO 1\r\n')
+        self.transport.clear()
+        self.send('250-PROTOCOLINFO 1')
+        self.send('250-AUTH METHODS=NULL')
+        self.send('250-VERSION Tor="0.2.2.34"')
+        self.send('250 OK')
+
+        self.assertEqual(self.transport.value(), 'AUTHENTICATE\r\n')
+
     def test_authenticate_password_deferred(self):
         d = defer.Deferred()
         self.protocol.password_function = lambda: d
@@ -335,6 +346,68 @@ OK''' % cookietmp.name)
             )
             self.assertTrue('AUTHENTICATE ' in self.transport.value())
 
+    def test_authenticate_cookie_without_reading(self):
+        server_nonce = str(bytearray([0] * 32))
+        server_hash = str(bytearray([0] * 32))
+        try:
+            self.protocol._safecookie_authchallenge(
+                '250 AUTHCHALLENGE SERVERHASH=%s SERVERNONCE=%s' %
+                (base64.b16encode(server_hash), base64.b16encode(server_nonce))
+            )
+            self.assertTrue(False)
+        except RuntimeError, e:
+            self.assertTrue('not read' in str(e))
+
+    def test_authenticate_unexisting_cookie_file(self):
+        unexisting_file = __file__ + "-unexisting"
+        try:
+            self.protocol._do_authenticate('''PROTOCOLINFO 1
+AUTH METHODS=COOKIE COOKIEFILE="%s"
+VERSION Tor="0.2.2.35"
+OK''' % unexisting_file)
+            self.assertTrue(False)
+        except RuntimeError:
+            pass
+
+    def test_authenticate_unexisting_safecookie_file(self):
+        unexisting_file = __file__ + "-unexisting"
+        try:
+            self.protocol._do_authenticate('''PROTOCOLINFO 1
+AUTH METHODS=SAFECOOKIE COOKIEFILE="%s"
+VERSION Tor="0.2.2.35"
+OK''' % unexisting_file)
+            self.assertTrue(False)
+        except RuntimeError:
+            pass
+
+    def test_authenticate_dont_send_cookiefile(self):
+        try:
+            self.protocol._do_authenticate('''PROTOCOLINFO 1
+AUTH METHODS=SAFECOOKIE
+VERSION Tor="0.2.2.35"
+OK''')
+            self.assertTrue(False)
+        except RuntimeError:
+            pass
+
+    def test_authenticate_password_when_cookie_unavailable(self):
+        unexisting_file = __file__ + "-unexisting"
+        self.protocol.password_function = lambda: 'foo'
+        self.protocol._do_authenticate('''PROTOCOLINFO 1
+AUTH METHODS=COOKIE,HASHEDPASSWORD COOKIEFILE="%s"
+VERSION Tor="0.2.2.35"
+OK''' % unexisting_file)
+        self.assertEqual(self.transport.value(), 'AUTHENTICATE %s\r\n' % "foo".encode("hex"))
+
+    def test_authenticate_password_when_safecookie_unavailable(self):
+        unexisting_file = __file__ + "-unexisting"
+        self.protocol.password_function = lambda: 'foo'
+        self.protocol._do_authenticate('''PROTOCOLINFO 1
+AUTH METHODS=SAFECOOKIE,HASHEDPASSWORD COOKIEFILE="%s"
+VERSION Tor="0.2.2.35"
+OK''' % unexisting_file)
+        self.assertEqual(self.transport.value(), 'AUTHENTICATE %s\r\n' % "foo".encode("hex"))
+
     def test_authenticate_safecookie_wrong_hash(self):
         cookiedata = str(bytearray([0] * 32))
         server_nonce = str(bytearray([0] * 32))
@@ -342,7 +415,7 @@ OK''' % cookietmp.name)
 
         # pretend we already did PROTOCOLINFO and read the cookie
         # file
-        self.protocol.cookie_data = cookiedata
+        self.protocol._cookie_data = cookiedata
         self.protocol.client_nonce = server_nonce  # all 0's anyway
         try:
             self.protocol._safecookie_authchallenge(
@@ -441,6 +514,18 @@ OK''' % cookietmp.name)
         self.send("250 OK")
         return d
 
+    def test_multiline_plus_embedded_equals(self):
+        """
+        """
+
+        d = self.protocol.get_info("FOO")
+        d.addCallback(CallbackChecker({"FOO": "\na="}))
+        self.send("250+FOO=")
+        self.send("a=")
+        self.send(".")
+        self.send("250 OK")
+        return d
+
     def incremental_check(self, expected, actual):
         if '=' in actual:
             return
@@ -648,6 +733,21 @@ iO3EUE0AEYah2W9gdz8t+i3Dtr0zgqLS841GC/TyDKCm+MKmN8d098qnwK0NGF9q
             self.send(line)
         return d
 
+    def test_getinfo_multiline(self):
+        descriptor_info = """250+desc/name/moria1=
+router moria1 128.31.0.34 9101 0 9131
+platform Tor 0.2.5.0-alpha-dev on Linux
+.
+250 OK"""
+        d = self.protocol.get_info("desc/name/moria1")
+        gold = "\nrouter moria1 128.31.0.34 9101 0 9131\nplatform Tor 0.2.5.0-alpha-dev on Linux"
+        d.addCallback(CallbackChecker({'desc/name/moria1': gold}))
+        d.addErrback(self.fail)
+
+        for line in descriptor_info.split('\n'):
+            self.send(line)
+        return d
+
     def test_addevent(self):
         self.protocol._set_valid_events('FOO BAR')
 
