@@ -57,7 +57,7 @@ class Stream(object):
         The ID of this stream, a number (or None if unset).
     """
 
-    def __init__(self, circuitcontainer):
+    def __init__(self, circuitcontainer, addrmap=None):
         """
         :param circuitcontainer: an object which implements
         :class:`interface.ICircuitContainer`
@@ -94,7 +94,7 @@ class Stream(object):
 
         self.listeners = []
         """A list of all connected
-        :class:`txtorcon.interface.ICircuitListener` instances."""
+        :class:`txtorcon.interface.IStreamListener` instances."""
 
         self.source_addr = None
         """If available, the address from which this Stream originated
@@ -111,6 +111,8 @@ class Stream(object):
         """Internal. Holds Deferred that will callback when this
         stream is CLOSED, FAILED (or DETACHED??)"""
 
+        self._addrmap = addrmap
+
     def listen(self, listen):
         """
         Attach an :class:`txtorcon.interface.IStreamListener` to this stream.
@@ -190,14 +192,23 @@ class Stream(object):
                 last_colon = args[3].rfind(':')
                 self.target_host = args[3][:last_colon]
                 self.target_port = int(args[3][last_colon + 1:])
+                # target_host is often an IP address (newer tors? did
+                # this change?) so we attempt to look it up in our
+                # AddrMap and make it a name no matter what.
+                if self._addrmap:
+                    try:
+                        h = self._addrmap.find(self.target_host)
+                        self.target_host = h.name
+                    except KeyError:
+                        pass
 
             self.target_port = int(self.target_port)
             if self.state == 'NEW':
                 if self.circuit is not None:
                     log.err(RuntimeError("Weird: circuit valid in NEW"))
-                [x.stream_new(self) for x in self.listeners]
+                self._notify('stream_new', self)
             else:
-                [x.stream_succeeded(self) for x in self.listeners]
+                self._notify('stream_succeeded', self)
 
         elif self.state == 'REMAP':
             self.target_addr = maybe_ip_addr(args[3][:args[3].rfind(':')])
@@ -208,7 +219,7 @@ class Stream(object):
             self.circuit = None
             self.maybe_call_closing_deferred()
             flags = self._create_flags(kw)
-            [x.stream_closed(self, **flags) for x in self.listeners]
+            self._notify('stream_closed', self, **flags)
 
         elif self.state == 'FAILED':
             if self.circuit:
@@ -217,7 +228,7 @@ class Stream(object):
             self.maybe_call_closing_deferred()
             # build lower-case version of all flags
             flags = self._create_flags(kw)
-            [x.stream_failed(self, **flags) for x in self.listeners]
+            self._notify('stream_failed', self, **flags)
 
         elif self.state == 'SENTCONNECT':
             pass  # print 'SENTCONNECT',self,args
@@ -230,7 +241,7 @@ class Stream(object):
             # FIXME does this count as closed?
             # self.maybe_call_closing_deferred()
             flags = self._create_flags(kw)
-            [x.stream_detach(self, **flags) for x in self.listeners]
+            self._notify('stream_detach', self, **flags)
 
         elif self.state in ['NEWRESOLVE', 'SENTRESOLVE']:
             pass  # print self.state, self, args
@@ -253,8 +264,7 @@ class Stream(object):
                     self.circuit = self.circuit_container.find_circuit(cid)
                     if self not in self.circuit.streams:
                         self.circuit.streams.append(self)
-                        for x in self.listeners:
-                            x.stream_attach(self, self.circuit)
+                        self._notify('stream_attach', self, self.circuit)
 
                 else:
                     if self.circuit.id != cid:
@@ -265,6 +275,17 @@ class Stream(object):
                             )
                         )
 
+    def _notify(self, func, *args, **kw):
+        """
+        Internal helper. Calls the IStreamListener function 'func' with
+        the given args, guarding around errors.
+        """
+        for x in self.listeners:
+            try:
+                getattr(x, func)(*args, **kw)
+            except Exception:
+                log.err()
+
     def maybe_call_closing_deferred(self):
         """
         Used internally to callback on the _closing_deferred if it
